function [secv,predcv,dof,lot,secv_stats] = cvxx(x,y,split,func,cent,varargin)

    // fonction cvxx_2 en dessous

    // Cross validation of any regression function
    //
    // Input:
    // =====
    // x:           matrix, n X p
    // y:           vector of reference values, n X 1
    // nf:          number of factors (latent variables for PLSR); usually an integer but it
    //              can be a vector for the 'ridge' regression
    // split:       how x is splitted during cross-validation
    //              if split is not a structure:
    //                  a scalar ns provokes a random ns subset CV
    //                  two scalars [ns,ni] provoke a double CV (ns random subsets) x ni iterations
    //                  a vector of n block numbers provokes a guided subset CV
    //                  a matrix of n x k block numbers provokes k guided subset CV
    //                  "vnbN" provokes a N blocks venitian blind CV (e.g. "vnb15")
    //                  "jckN" provokes a N blocks jackkniffe CV (e.g. "jck15")
    //              if split is a structure:
    //                  .method:        a string        the CV method: 'random','jack','venitian' or 'prefixed'
    //                  .nblocks:       the parameter attached to the method
    //                                  'random', 'jack' or 'venitian':'    -> the number of blocks
    //                                  'prefixed -> a conjunctive matrix (n*r) with the numbers of the k blocks repeated r times
    //                                               or a disjunctive matrix (n * k * r) for k blocks repeated r times
    //                  .nbrepeat:      the number of repetitions of the CV
    //                  .sameobs:       the identification of the observations that cannot be split in cal and val datasets
    // func:        function called for each step of the cross validation
    //              possible choices: 'pls' , 'pcr', 'vodka' ,'ridge'
    // cent :       if 1, data are centered
    //              if 0, data are not centered
    //
    //
    //
    // Output:
    // =======
    // secv:    cross validation errors. The size depends on the parameters of the function used (ex: nbr of LV)
    // predcv:  predictions of the cross validation
    // 
    // Example:
    // ========
    // [secv,pred] = xxcvn(x,y,[10 30],'pls',1 ,20); 
    //          cross validation applied to 10 blocks 30 times for a PLSR on x,y with 20 
    //          latent variables and with centering 
    // [secv,pred] = xxcvn(x,y,10,'vodka',20, 2,1); 
    //          cross validation applied to 10 blocks for a VODKA-PLSR on x,y with 20 
    //          latent variables, r option=2 and after centering 
    // 
    // Miscellaneous:
    // ==============
    // JM Roger (IRSTEA) and JC Boulet (INRA), France
    // Last update: 2018, Novembre, 5
    // -----------------------------------------------  


    // ------------------------------------------------------------------retro-compatibilité:
    if typeof(split)~='st' then
        
        [secv,predcv,dof,all_bcoeffs] = cvxx_2(x,y,split,func,cent,varargin(:));   // fonction en dessous de celle-ci 

        lot=[];

//        // calcul de la variabilité des b-coeffs      //1mars21 --- mis en commentaire car pas utilisé 
//        [ntemp,ptemp,qtemp]=size(all_bcoeffs);
//        var_bcoeffs=zeros(ntemp,ptemp);
//        for i=1:ptemp;
//            b_temp=all_bcoeffs(:,i,:);
//            b_temp=matrix(b_temp,[ntemp qtemp]);
//            b_temp_stdev=std(b_temp');
//            var_bcoeffs(:,i)=b_temp_stdev';
//        end
//        // var_bcoeffs est de dimensions (q x lv)  q=nbre de variables --------------------------------

        // calcul des écarts de prédiction 
        [nx,px]=size(x);
        [ntemp,ptemp,qtemp]=size(all_bcoeffs);
        xc=x-cent*ones(nx,1)*mean(x,'r');
        yc=y-cent*ones(nx,1)*mean(y,'r');
        
        rmsecv_all=zeros(ptemp,qtemp);
        for i=1:ptemp;
            for j=1:qtemp;
                err_pred=xc*all_bcoeffs(:,i,j)+cent*ones(nx,1)*mean(y)-y;
                rmsecv_all(i,j)=sqrt(err_pred'*err_pred/nx);
            end;
        end;

        // calcul des écarts-type 
        secv_stats.mean=mean(rmsecv_all,'c');
        secv_stats.stdev=std(rmsecv_all')';

    // --------------------------------------------------------------------------------------
    else
        [mx,nx] = size(x);
        [my,ny] = size(y);
        if mx ~= my
            error('The number of samples of x and y does not match')
        end


        // -------------------------------------------------------------
        // gestion des arguments, champs et valeurs par défaut de split
        split2=split;

        if isfield(split2,'method') then
            method=split2.method;
        else
            method='random';
            split2.method='random';
        end

        if isfield(split2,'nblocks') then
            nblocks=split2.nblocks;
        else
            nblocs=2;
            split2.nblocks=2;
        end

        if isfield(split2,'nbrepeat') then
            if method ~='prefixed' then
                nbrepeat=split2.nbrepeat;
            else
                nbrepeat=size(split2.nbrepeat,2);               // nbre de répétitions = nbre de colonne
                split2.nbrepeat=nbrepeat;
            end
        else
            nbrepeat=10;
        end

        if isfield(split2,'sameobs') then
            sameobs=split2.sameobs;
        else
            sameobs=[1:mx];                 // chaque observation est unique
            split2.sameobs=sameobs;
        end

        // validation des valeurs entrées:
        if method~='random' & method~='jack' & method~='venitian' & method~='prefixed' then
            error('CV method should be ', 'random'', ''jack'' ,''venitian'' or ''prefixed''.')
        end


        // ----------------------------------------------------------------------
        // utilisation des blocs en disjonctif
        lot=cvgroups(mx,split2);


        //----------------------------------------------------------------------
        // application de la CV avec les modèles
        [n,ni,nj] = size(lot);                                  // n=mx...

        for j=1:nj;                                             // pour chaque répétition
            for k=1:ni                                          // pour chaque lot
                ns = max(lot(:,k));                             // nombre de lots
                for i=1:ns
                    // splitting cal and test sets
                    indval =  find(lot(:,k)==i);
                    indcal =  find(lot(:,k)~=i);

                    // Calling of the calibration+test function:
                    if ~isempty(indval) &  ~isempty(indcal) then
                        [s,pr,biaisjc,last_argout]=caltestxx(x(indcal,:),y(indcal,:),x(indval,:),y(indval,:),func,cent,varargin(:));
                        dof=last_argout.dof;
                        clear biaisjc bcoeffjc tscoresjc ploadsjc  last_argout // dof is kept
                    end
                

                    if (i==1 & k==1 & j==1)  then                      // 1° appel -> initialisation  CONTINUER
                        press = 0*s;
                        sz = size(pr); 
                        sz=sz(2:$);
                        prcv = zeros(mx,prod(sz));

                        press_all=zeros(nj,size(s,1));               // nj =nbr repet  size(s,1)=nbr var. latentes   // 19aout21

                    end;
                    // the following works whatever the size of the arguments returned by caltestxx
                    press = press + (s.^2) * length(indval);
                    prcv(indval,:) = prcv(indval,:) + matrix(pr(:),length(indval),prod(sz));

                    press_all(nj,:)=press_all(nj,:) + (s.^2)' * length(indval);                                     //19aout21

                end
            end;
        end;

        // reshape of the output arguments
        secv = sqrt(press/(mx*ni*nj));
        predcv = matrix(prcv(:)./ni,[mx,sz]);

        secv_all=sqrt(press_all/(mx*ni));                                   //19aout21
        secv_stats.mean=mean(secv_all,'r')';
        secv_stats.stdev=stdev(secv_all,'r')';
        // verifie: si on somme press_all on obtient press
        // mais aussi: secv = secv_stats.stdev : tres surprenant


    end;

endfunction







// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX







function [secv,predcv,dof,all_bcoeffs] = cvxx_2(x,y,split,func,cent,varargin)

// function [secv,predcv] = cal_xxcvn(x,y,split,'func',(r -for vodka-),centering);
// Cross validation of any regression function
//
// Input:
// =====
// x:       matrix, n X p
// y:       vector of reference values, n X 1
// nf:      number of factors (latent variables for PLSR); usually an integer but it
//          can be a vector for the 'ridge' regression
// split:   how x is splitted during cross-validation
//            a scalar ns provokes a random ns subset CV
//            two scalars [ns,ni] provoke a double CV (ns random subsets) x ni iterations
//            a vector of n block numbers provokes a guided subset CV
//            a matrix of n x k block numbers provokes k guided subset CV
//            "vnbN" provokes a N blocks venitian blind CV (e.g. "vnb15")
//            "jckN" provokes a N blocks jackkniffe CV (e.g. "jck15")
// 'func':  function called for each step of the cross validation
//            possible choices: 'pls' , 'pcr', 'vodka' ,'ridge'
// cent :  if 1, data are centered
//         if 0, data are not centered
//
// Output:
// =======
// secv:    cross validation errors. The size depends on the function used
// predcv:  predictions of the cross validation
//
// Example:
// ========
// [secv,pred] = xxcvn(x,y,[10 30],'pls',1 ,20);
//          cross validation applied to 10 blocks 30 times for a PLSR on x,y with 20
//          latent variables and with centering
// [secv,pred] = xxcvn(x,y,10,'vodka',20, 2,1);
//          cross validation applied to 10 blocks for a VODKA-PLSR on x,y with 20
//          latent variables, r option=2 and after centering
//
// Miscellaneous:
// ==============
// JM Roger (IRSTEA) and JC Boulet (INRA), France
// Last update: 2012, March, 28
// -----------------------------------------------

    [mx,nx] = size(x);
    [my,ny] = size(y);
    if mx ~= my
        error('The number of samples of x and y does not match')
    end

    if part(func,[1 2])=='nn' then
        flag='NL';
    else
    flag='L';
    end

    //lv=varargin(1);         // 26fev21. Supprime le 18aout21
    



    // determination of the blocks and iterations
    lot=cvgroups(mx,split);

    [n,ni] = size(lot);

    press = 0;
    
    all_bcoeffs=[];        //26fev21
    index=1;

    for k=1 : ni    // for each repetition
        ns = max(lot(:,k)); // number of splits
        for i=1 : ns
            // splitting cal and test sets
            indval =  find(lot(:,k)==i);
            indcal =  find(lot(:,k)~=i);

            // Calling of the calibration+test function:
            if ~isempty(indval) &  ~isempty(indcal) & flag=='L' then  //correction v.0.6
                [s,pr,biaisjc,last_argout]=caltestxx(x(indcal,:),y(indcal,:),x(indval,:),y(indval,:),func,cent,varargin(:));
                dof=last_argout.dof;
                
                if all_bcoeffs==[] then                             //27fev21
                    all_bcoeffs(:,:,1)=last_argout.bcoeffs;
                else
                    ntemp=size(all_bcoeffs,3);
                    all_bcoeffs(:,:,ntemp+1)=last_argout.bcoeffs;
                end;
                
                clear biaisjc bcoeffjc tscoresjc ploadsjc  last_argout // dof is kept
            elseif ~isempty(indval) &  ~isempty(indcal) & flag=='NL' then
                [s,pr]=caltestxx(x(indcal,:),y(indcal,:),x(indval,:),y(indval,:),func,cent,varargin(:));
                dof=[];
            end


            if (i==1 & k==1) | ~isdef('prcv') then // if first call
                press = s.*0;
                sz = size(pr);
                sz=sz(2:$);
                prcv = zeros(mx,prod(sz));
            end;
            // the following works whatever the size of the arguments returned by caltestxx
            press = press + (s.^2) * length(indval);
            prcv(indval,:) = prcv(indval,:) + matrix(pr(:),length(indval),prod(sz));
            
            index=index+1;

        end
    end;

    // reshape of the output arguments
    secv = sqrt(press/(mx*ni));
    predcv = matrix(prcv(:)./ni,[mx,sz]);

endfunction



